package com.compdfkit.pdfviewer.activities;

import androidx.annotation.NonNull;
import androidx.appcompat.app.AppCompatActivity;
import androidx.appcompat.widget.Toolbar;
import androidx.core.view.MenuProvider;
import androidx.core.view.ViewCompat;
import androidx.core.view.WindowInsetsControllerCompat;

import android.graphics.Bitmap;
import android.graphics.RectF;
import android.net.Uri;
import android.os.Bundle;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.widget.ImageView;
import android.widget.Toast;

import com.compdfkit.core.common.CPDFDocumentException;
import com.compdfkit.core.document.CPDFBackground;
import com.compdfkit.core.document.CPDFDocument;
import com.compdfkit.core.page.CPDFPage;
import com.compdfkit.pdfviewer.R;
import com.compdfkit.pdfviewer.entity.BackgroundSetting;
import com.compdfkit.pdfviewer.tools.DialogManager;

import java.io.File;
import java.util.Random;

public class BackgroundActivity extends AppCompatActivity implements MenuProvider {
    private int pageCount = 0;

    private View pageContainer;
    private ImageView pageImage;

    private int currentPageHeight = 0;
    private int currentPageWidth = 0;

    private int imageHeight;
    private int imageWidth;

    private CPDFDocument document = new CPDFDocument(this);

    private float sizeConvertRate = 0;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        int[] themes = new int[] {R.style.AppTheme, R.style.AppRedTheme, R.style.AppOrangeTheme, R.style.AppGreenTheme, R.style.AppCyanBlueTheme};
        int randomTheme = new Random().nextInt(5);
        setTheme(themes[randomTheme]);

        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_background);

        Toolbar toolbar = findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);
        addMenuProvider(this);

        File PDFFile = new File("/storage/emulated/0/TestPDFs/test.pdf");
        Uri PDFUri = Uri.fromFile(PDFFile);

        reloadDocument(PDFFile, PDFUri);

        pageContainer = findViewById(R.id.page_container);

        pageImage = findViewById(R.id.page_image);
        pageImage.post(this::initPageImage);

        if (this.getResources().getConfiguration().uiMode == 0x11) {
            WindowInsetsControllerCompat wic = ViewCompat.getWindowInsetsController(getWindow().getDecorView());
            if (wic != null)
                wic.setAppearanceLightStatusBars(true);
        }
    }

    @Override
    public void onCreateMenu(@NonNull Menu menu, @NonNull MenuInflater menuInflater) {
        menuInflater.inflate(R.menu.menu_background, menu);
    }

    @Override
    public boolean onMenuItemSelected(@NonNull MenuItem menuItem) {
        return false;
    }

    private void initPageImage() {
        CPDFPage currentPage = document.pageAtIndex(0);

        RectF pageSize = document.getPageSize(0);
        float pageW = pageSize.right - pageSize.left;
        float pageH = pageSize.bottom - pageSize.top;

        if (pageW / pageH >  ((float) pageContainer.getWidth() / (float) pageContainer.getHeight())) {
            sizeConvertRate = pageContainer.getWidth() / pageW;

            imageHeight = (int) (sizeConvertRate * pageH);
            imageWidth = pageContainer.getWidth();

            pageImage.setMaxHeight(imageHeight);
            pageImage.setMaxWidth(imageWidth);
        } else {
            sizeConvertRate = pageContainer.getHeight() / pageH;

            imageHeight = pageContainer.getHeight();
            imageWidth = (int) (sizeConvertRate * pageW);

            pageImage.setMaxWidth(imageWidth);
        }

        currentPageHeight = imageHeight;
        currentPageWidth = imageWidth;
        Bitmap pageBitmap = Bitmap.createBitmap(imageWidth, imageHeight, Bitmap.Config.ARGB_4444);

        currentPage.renderPage(pageBitmap, 0, 0, imageWidth, imageHeight, 0xFFFFFFFF, 255, 0, true, true);
        refreshPageImage(pageBitmap);
    }

    public void reloadDocument(@NonNull File PDFFile, Uri PDFUri) {
        CPDFDocument.PDFDocumentError error = document.open(PDFFile.getAbsolutePath());

        if (error == CPDFDocument.PDFDocumentError.PDFDocumentErrorPassword) {
            error = document.open(PDFUri, "password");
        } else if (error != CPDFDocument.PDFDocumentError.PDFDocumentErrorSuccess) {
            Toast.makeText(this, "File failed.", Toast.LENGTH_SHORT).show();
        } else {
            pageCount = document.getPageCount();
        }
    }

    //TODO: Replace the same code.
    public void reloadDocument() throws CPDFDocumentException {
        if (document.hasChanges()) {
            boolean res;
            res = document.save();
        }
        document.close();

        File PDFFile = new File("/storage/emulated/0/TestPDFs/test.pdf");
        Uri PDFUri = Uri.fromFile(PDFFile);

        reloadDocument(PDFFile, PDFUri);
        CPDFPage currentPage = document.pageAtIndex(0);

        Bitmap pageBitmap = Bitmap.createBitmap(currentPageWidth, currentPageHeight, Bitmap.Config.ARGB_4444);
        currentPage.renderPage(pageBitmap, 0, 0, currentPageWidth, currentPageHeight, 0xFFFFFFFF, 255, 0, true, true);
        refreshPageImage(pageBitmap);
    }

    public void refreshPageImage(Bitmap updatePageBitmap) {
        pageImage.setImageBitmap(updatePageBitmap);
    }

    public void showSetting(MenuItem menuItem) {
        DialogManager.showBackgroundSettingDialog(this);
    }

    public void finishEdit(MenuItem menuItem) throws CPDFDocumentException {
        CPDFBackground background = document.getBackground();

        BackgroundSetting setting = new BackgroundSetting();
        setting.setColor(background.getColor());
        setting.setOpacity(background.getOpacity());
        setting.setRotation(background.getRotation());
        setting.setxOffset(background.getXOffset());
        setting.setyOffset(background.getYOffset());
        setting.setPages(background.getPages());
        setting.setScale(background.getScale());

        background.setColor(0xFF66CCFF);
        background.setOpacity(0.f);
        background.setRotation(0.f);
        background.setXOffset(0.f);
        background.setPages("0-4");
        background.setScale(1.f);
        background.update();

        reloadDocument();
    }
}