package com.compdfkit.pdfviewer.activities;

import static android.content.ContentValues.TAG;

import androidx.annotation.NonNull;
import androidx.appcompat.app.AppCompatActivity;
import androidx.appcompat.widget.Toolbar;
import androidx.core.view.MenuProvider;
import androidx.core.view.ViewCompat;
import androidx.core.view.WindowInsetsControllerCompat;

import android.content.Intent;
import android.graphics.Bitmap;
import android.graphics.RectF;
import android.net.Uri;
import android.os.Build;
import android.os.Bundle;
import android.os.Environment;
import android.provider.Settings;
import android.util.Log;
import android.view.DragEvent;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.view.WindowManager;
import android.widget.ImageView;
import android.widget.Toast;

import com.compdfkit.core.common.CPDFDocumentException;
import com.compdfkit.core.document.CPDFDocument;
import com.compdfkit.core.page.CPDFPage;
import com.compdfkit.core.watermark.CPDFImageWatermark;
import com.compdfkit.core.watermark.CPDFWatermark;
import com.compdfkit.pdfviewer.R;
import com.compdfkit.pdfviewer.customview.listview.WatermarkListView;
import com.compdfkit.pdfviewer.customview.watermarkview.ImageWatermarkView;
import com.compdfkit.pdfviewer.customview.watermarkview.TextWatermarkView;
import com.compdfkit.pdfviewer.customview.watermarkview.WatermarkView;
import com.compdfkit.pdfviewer.entity.WatermarkSetting;
import com.compdfkit.pdfviewer.fragments.WatermarkConsoleFragment;
import com.compdfkit.pdfviewer.tools.ConvertMapper;
import com.compdfkit.pdfviewer.tools.WatermarkHandle;
import com.google.android.material.tabs.TabLayout;

import java.io.File;
import java.util.Objects;
import java.util.Random;

public class WatermarkActivity extends AppCompatActivity implements MenuProvider {
    private TabLayout tabLayout;
    private View pageContainer;

    private ImageView pageImage;
    private ImageView fullScreenMask;

    private WatermarkListView watermarkListView;

    private CPDFDocument document = new CPDFDocument(this);
    private WatermarkView watermark;
    private WatermarkConsoleFragment consoleFragment;

    private MenuItem doneItem;

    private int currentPageHeight = 0;
    private int currentPageWidth = 0;

    private int imageHeight;
    private int imageWidth;

    private int cacheIndex = -1;

    private boolean editMode = false;

    private WatermarkSetting watermarkSettingCache;

    private float sizeConvertRate = 0;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        int[] themes = new int[] {R.style.AppTheme, R.style.AppRedTheme, R.style.AppOrangeTheme, R.style.AppGreenTheme, R.style.AppCyanBlueTheme};
        int randomTheme = new Random().nextInt(5);
        setTheme(themes[randomTheme]);

        super.onCreate(savedInstanceState);

        WindowManager.LayoutParams windowAttributes = getWindow().getAttributes();
        windowAttributes.rotationAnimation = WindowManager.LayoutParams.ROTATION_ANIMATION_CROSSFADE;
        getWindow().setAttributes(windowAttributes);

        setContentView(R.layout.activity_watermark);
        addMenuProvider(this);

        if (this.getResources().getConfiguration().uiMode == 0x11) {
            WindowInsetsControllerCompat wic = ViewCompat.getWindowInsetsController(getWindow().getDecorView());
            if (wic != null)
                wic.setAppearanceLightStatusBars(true);
        }

        Toolbar toolbar = findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);

        pageContainer = findViewById(R.id.page_container);

        watermark = findViewById(R.id.watermark_text);

        consoleFragment = (WatermarkConsoleFragment) getSupportFragmentManager().findFragmentByTag("console_fragment");
        assert consoleFragment != null;
        consoleFragment.setAssociatedWatermark(watermark);

        pageImage = findViewById(R.id.page_image);
        fullScreenMask = findViewById(R.id.full_screen_mask);

        watermarkListView = findViewById(R.id.watermark_list_view);

        tabLayout = findViewById(R.id.tab_layout);
        initTabLayout();

        if (!(Build.VERSION.SDK_INT < Build.VERSION_CODES.R || Environment.isExternalStorageManager())) {
            Intent intent = new Intent(Settings.ACTION_MANAGE_ALL_FILES_ACCESS_PERMISSION);
            intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
            startActivity(intent);
        }

        File PDFFile = new File("/storage/emulated/0/TestPDFs/test.pdf");
        Uri PDFUri = Uri.fromFile(PDFFile);

        reloadDocument(PDFFile, PDFUri);

        consoleFragment.setDocumentPageCount(document.getPageCount());

        pageImage.post(() -> {
            initPageImageContent();
            initPageImageListener();
        });

        watermarkListView.post(() -> {
            watermarkListView.scrollTo(watermarkListView.getWidth(), 0);
            watermarkListView.initList(document, consoleFragment);
        });

        Log.d("Render. Watermark Count", "current watermark count:" + document.getWatermarkCount());
    }

    @Override
    public void onCreateMenu(@NonNull Menu menu, @NonNull MenuInflater menuInflater) {
        menuInflater.inflate(R.menu.menu_watermark, menu);

        doneItem = menu.findItem(R.id.done);
    }

    @Override
    public boolean onMenuItemSelected(@NonNull MenuItem menuItem) {
        return false;
    }

    public void initTabLayout() {

        tabLayout.addOnTabSelectedListener(new TabLayout.OnTabSelectedListener() {
            @Override
            public void onTabSelected(TabLayout.Tab tab) {
                changeWatermark(tab.getPosition());
            }

            @Override
            public void onTabUnselected(TabLayout.Tab tab) {}

            @Override
            public void onTabReselected(TabLayout.Tab tab) {}
        });
    }

    public void initPageImageContent() {
        CPDFPage currentPage = document.pageAtIndex(0);

        RectF pageSize = document.getPageSize(0);
        float pageW = pageSize.right - pageSize.left;
        float pageH = pageSize.bottom - pageSize.top;

        if (pageW / pageH >  ((float) pageContainer.getWidth() / (float) pageContainer.getHeight())) {
            sizeConvertRate = pageContainer.getWidth() / pageW;

            imageHeight = (int) (sizeConvertRate * pageH);
            imageWidth = pageContainer.getWidth();

            pageImage.setMaxHeight(imageHeight);
            pageImage.setMaxWidth(imageWidth);
        } else {
            sizeConvertRate = pageContainer.getHeight() / pageH;

            imageHeight = pageContainer.getHeight();
            imageWidth = (int) (sizeConvertRate * pageW);

            pageImage.setMaxWidth(imageWidth);
        }

        TextWatermarkView tempT = (TextWatermarkView) watermark;
        tempT.setSizeConvertRate(sizeConvertRate);
        ImageWatermarkView tempI = findViewById(R.id.watermark_image);
        tempI.setSizeConvertRate(sizeConvertRate);

        currentPageHeight = imageHeight;
        currentPageWidth = imageWidth;
        Bitmap pageBitmap = Bitmap.createBitmap(imageWidth, imageHeight, Bitmap.Config.ARGB_4444);

        currentPage.renderPage(pageBitmap, 0, 0, imageWidth, imageHeight, 0xFFFFFFFF, 255, 0, true, true);
        refreshPageImage(pageBitmap);
        fullScreenMask.setImageBitmap(pageBitmap);
    }

    public void initPageImageListener() {
        pageImage.setOnDragListener((view, dragEvent) -> {
            float xOffset = 1.0f * watermark.getWidth() / 2;
            float yOffset = 1.0f * watermark.getHeight() / 2;

            if (watermark.getCurrentEvent() == WatermarkView.DRAG_EVENT) {
                return handleDragEvent(dragEvent, xOffset, yOffset);
            } else {
                return handleRollEvent(dragEvent, xOffset, yOffset);
            }
        });
    }

    public void setTabLayoutEnable(boolean enable) {
        ViewGroup viewGroup = (ViewGroup) tabLayout.getChildAt(0);;
        if (viewGroup != null) {
            for (int childIndex = 0; childIndex < viewGroup.getChildCount(); childIndex++) {
                View tabView = viewGroup.getChildAt(childIndex);
                if (tabView != null)
                    tabView.setEnabled(enable);
            }
        }
    }

    public boolean handleDragEvent(@NonNull DragEvent dragEvent, float xOffset, float yOffset) {
        int xOutline = pageImage.getLeft();
        int yOutline = pageImage.getTop();

        switch (dragEvent.getAction()) {
            case DragEvent.ACTION_DRAG_LOCATION:
                watermark.setX(dragEvent.getX() + xOutline - xOffset);
                watermark.setY(dragEvent.getY() + yOutline - yOffset);

                watermark.setPDFXOffset(ConvertMapper.convertViewXToPDFXOffset(watermark.getX() - pageImage.getLeft(), xOffset, imageWidth / 2f, sizeConvertRate));
                watermark.setPDFYOffset(ConvertMapper.convertViewYToPDFYOffset(watermark.getY() - pageImage.getTop(), yOffset, imageHeight / 2f, sizeConvertRate));

                watermark.setViewXOffset(watermark.getPDFXOffset() * sizeConvertRate);
                watermark.setViewYOffset(watermark.getPDFYOffset() * sizeConvertRate);

                return true;
            case DragEvent.ACTION_DRAG_ENDED:
                watermark.setVisibility(View.VISIBLE);
                return false;
            default:
                return true;
        }
    }

    public boolean handleRollEvent(@NonNull DragEvent dragEvent, float xOffset, float yOffset) {
        float currentSlope;

        double currentRotation = 0;
        double resultRotation = 0;

        float baseSlope = yOffset / xOffset * -1;

        float centerPointX = watermark.getX() + 1.0f * watermark.getWidth() / 2; /* 控件中心点x = 当前控件位置x + 控件一半宽度 */
        float centerPointY = watermark.getY() + 1.0f * watermark.getHeight() / 2; /* 控件中心点y = 当前控件位置y + 控件一半高度 */

        if (dragEvent.getAction() == DragEvent.ACTION_DRAG_LOCATION) {
            currentSlope = (dragEvent.getY() + pageImage.getTop() - centerPointY) / (dragEvent.getX() + pageImage.getLeft() - centerPointX) * -1;
            currentRotation = Math.toDegrees(Math.atan(currentSlope));

            if ((dragEvent.getY() + pageImage.getTop() < centerPointY && dragEvent.getX() + pageImage.getLeft() < centerPointX) || (dragEvent.getY() + pageImage.getTop() > centerPointY && dragEvent.getX() + pageImage.getLeft() < centerPointX))
                currentRotation += 180;

            resultRotation = currentRotation - Math.toDegrees(Math.atan(baseSlope));

            watermark.setRotation((float) resultRotation * -1);
            watermark.setRotationRadian((float) Math.toRadians(resultRotation));

            return true;
        }
        return true;
    }

    public void changeWatermarkLocation(float x, float y) {
        float xOffset = 1.0f * watermark.getWidth() / 2;
        float yOffset = 1.0f * watermark.getHeight() / 2;

        watermark.setX(ConvertMapper.convertPDFXOffsetToViewX(x, xOffset, imageWidth / 2f, sizeConvertRate) + pageImage.getLeft());
        watermark.setY(ConvertMapper.convertPDFYOffsetToViewY(y, yOffset, imageHeight / 2f, sizeConvertRate) + pageImage.getTop());

        watermark.setPDFXOffset(x);
        watermark.setPDFYOffset(y);

        watermark.setViewXOffset(x * sizeConvertRate);
        watermark.setViewYOffset(y * sizeConvertRate);
    }

    public void changeWatermark(int type) {
        watermark.setVisibility(View.INVISIBLE);
        if (type == 1) {
            watermark = findViewById(R.id.watermark_image);
        } else {
            watermark = findViewById(R.id.watermark_text);
        }
        watermark.setVisibility(View.VISIBLE);

        consoleFragment.setAssociatedWatermark(watermark);
        consoleFragment.changePicker(type);
    }

    public void refreshPageImage(Bitmap updatePageBitmap) {
        pageImage.setImageBitmap(updatePageBitmap);
    }

    public void reloadDocument() throws CPDFDocumentException {
        if (document.hasChanges()) {
            boolean res;
            res = document.save();
        }
        document.close();

        File PDFFile = new File("/storage/emulated/0/TestPDFs/test.pdf");
        Uri PDFUri = Uri.fromFile(PDFFile);

        reloadDocument(PDFFile, PDFUri);
        CPDFPage currentPage = document.pageAtIndex(0);

        Bitmap pageBitmap = Bitmap.createBitmap(currentPageWidth, currentPageHeight, Bitmap.Config.ARGB_4444);
        currentPage.renderPage(pageBitmap, 0, 0, currentPageWidth, currentPageHeight, 0xFFFFFFFF, 255, 0, true, true);
        refreshPageImage(pageBitmap);
    }

    public void reloadDocument(@NonNull File PDFFile, Uri PDFUri) {
        CPDFDocument.PDFDocumentError error = document.open(PDFFile.getAbsolutePath());

        if (error == CPDFDocument.PDFDocumentError.PDFDocumentErrorPassword) {
            error = document.open(PDFUri, "password");
        } else if (error != CPDFDocument.PDFDocumentError.PDFDocumentErrorSuccess) {
            Toast.makeText(this, "File failed.", Toast.LENGTH_SHORT).show();
        }
    }

    public void createNewWatermark(WatermarkSetting setting) {
        CPDFWatermark.Type[] types = {CPDFWatermark.Type.WATERMARK_TYPE_TEXT, CPDFWatermark.Type.WATERMARK_TYPE_IMG};
        CPDFWatermark currentWatermark = document.createWatermark(types[consoleFragment.getMode()]);

        WatermarkHandle.updateWatermark(currentWatermark, setting, sizeConvertRate);
        watermarkListView.addSettingIntoList(setting);
    }

    public void changeWatermark(CPDFWatermark watermark, WatermarkSetting setting) {
        WatermarkHandle.updateWatermark(watermark, setting, sizeConvertRate);
//        adapter.updateSettingList(cacheIndex, setting);
    }

    public void deleteWatermark(int deleteIndex) throws CPDFDocumentException {
        CPDFWatermark targetWatermark = document.getWatermark(deleteIndex);
        targetWatermark.clear();

        reloadDocument();
    }

    public void resetWatermark(int index) {
        if (index == -1)
            return;

        CPDFWatermark tempWatermark = document.getWatermark(index);
        WatermarkHandle.updateWatermark(tempWatermark, watermarkSettingCache, sizeConvertRate);
    }

    public void hideWatermark(int index) throws CPDFDocumentException {
        CPDFWatermark currentWatermark = document.getWatermark(index);
        if (index == cacheIndex)
            return;

        resetWatermark(cacheIndex);
        cacheIndex = index;
        watermarkSettingCache = WatermarkHandle.getSettingFromList(index);

        if (currentWatermark.getType() == CPDFWatermark.Type.WATERMARK_TYPE_IMG) {
            CPDFImageWatermark imageWatermark = (CPDFImageWatermark) currentWatermark;
            imageWatermark.setOpacity(0f);
            imageWatermark.update();
        } else {
            currentWatermark.setOpacity(0f);
            currentWatermark.update();
        }

        Log.d(TAG, "hideWatermark: current watermark count:" + document.getWatermarkCount());

        doneItem.setEnabled(true);

        reloadDocument();
    }

    public void finishEditWatermark(MenuItem menuItem) throws CPDFDocumentException {
        WatermarkSetting setting = consoleFragment.getWatermarkSetting();
        watermark.completeSetting(setting);

        if (!editMode) {
            createNewWatermark(setting);
            Toast.makeText(this, "Add new watermark", Toast.LENGTH_SHORT).show();

            reloadDocument();
        } else {
            changeWatermark(document.getWatermark(cacheIndex), setting);
            Toast.makeText(this, "Change a watermark, index:" + cacheIndex, Toast.LENGTH_SHORT).show();

            cacheIndex = -1;

            expandWatermarkList();
        }
        Log.d("Finish. Watermark Count", "current watermark count:" + document.getWatermarkCount());
    }

    public void showFullScreenMask(boolean isFullScreen) {
        if (isFullScreen) {
            int imageHeight = pageImage.getHeight();
            int imageWidth = pageImage.getWidth();

            WatermarkView temp = watermark;

            Bitmap fullScreenBitmap = temp.getFullScreenBitmap(imageWidth, imageHeight,
                    (int) ConvertMapper.convertMMtoViewUnit(consoleFragment.gethSpace(), sizeConvertRate),
                    (int) ConvertMapper.convertMMtoViewUnit(consoleFragment.getvSpace(), sizeConvertRate),
                    (int) watermark.getViewXOffset(),
                    (int) watermark.getViewYOffset(),
                    watermark.getRotation());
            fullScreenMask.setImageBitmap(fullScreenBitmap);
            fullScreenMask.setAlpha(consoleFragment.getCurrentOpacity());
            fullScreenMask.setVisibility(View.VISIBLE);
            watermark.setVisibility(View.INVISIBLE);
        } else {
            fullScreenMask.setVisibility(View.INVISIBLE);
            watermark.setVisibility(View.VISIBLE);
        }
    }

    public void changeEditMode(MenuItem menuItem) {
        editMode = !editMode;

        Objects.requireNonNull(getSupportActionBar()).setTitle(editMode ? R.string.edit_watermark : R.string.add_watermark);
        menuItem.setIcon(editMode ? R.drawable.ic_baseline_watermark_add_24 : R.drawable.ic_baseline_edit_note_24);

        watermark.setVisibility(editMode ? View.INVISIBLE : View.VISIBLE);
        if (editMode) {
            watermarkListView.showView();

            setTabLayoutEnable(false);
            doneItem.setEnabled(false);
        } else {
            resetWatermark(cacheIndex);
            watermarkListView.hintView();

            setTabLayoutEnable(true);
            doneItem.setEnabled(true);
        }
    }

    public void expandWatermarkList() throws CPDFDocumentException {
        watermarkListView.showView();

        resetWatermark(cacheIndex);
        watermark.setVisibility(View.INVISIBLE);
        reloadDocument();

        doneItem.setEnabled(false);
    }

    public void closeWatermarkList() {
        watermarkListView.hintList();
        watermark.setVisibility(View.VISIBLE);
    }
}