package com.compdfkit.pdfviewer.customview.picker;

import android.content.Context;
import android.util.AttributeSet;
import android.view.LayoutInflater;
import android.view.View;
import android.widget.ImageView;
import android.widget.Scroller;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.constraintlayout.widget.ConstraintLayout;

import com.compdfkit.pdfviewer.R;
import com.compdfkit.pdfviewer.customview.watermarkview.TextWatermarkView;

public class ColorPicker extends ConstraintLayout {
    private View currentView;
    private LayoutInflater inflater;
    private Context currentContext;

    private Scroller mScroller;

    private TextWatermarkView textWatermark;

    private final int[] colorPickers = { R.id.color_1, R.id.color_2, R.id.color_3, R.id.color_4, R.id.color_5, R.id.color_6, R.id.color_7 };
    private final int[] colors = { 0xFFFF0000, 0xFFFF6600, 0xFFFFDD00, 0xFF00FF33, 0xFF00FFFF, 0xFF000000, 0xFF8800FF };

    public ColorPicker(@NonNull Context context) {
        this(context, null);
    }

    public ColorPicker(@NonNull Context context, @Nullable AttributeSet attrs) {
        this(context, attrs, 0);
    }

    public ColorPicker(@NonNull Context context, @Nullable AttributeSet attrs, int defStyleAttr) {
        this(context, attrs,defStyleAttr, 0);
    }

    public ColorPicker(@NonNull Context context, @Nullable AttributeSet attrs, int defStyleAttr, int defStyleRes) {
        super(context, attrs, defStyleAttr, defStyleRes);
        this.mScroller = new Scroller(context);
        init(context);
    }

    @Override
    public void computeScroll() {
        super.computeScroll();
        if (mScroller.computeScrollOffset()) {
            scrollTo(mScroller.getCurrX(), mScroller.getCurrY());
            invalidate();
        }
    }

    public void smoothScrollX(int destX) {
        int scrollX = getScrollX();
        int deltaX = destX - scrollX;
        mScroller.startScroll(scrollX, 0, deltaX, 0, 500);
        invalidate();
    }

    private void init(Context context) {
        mScroller = new Scroller(context);

        currentContext = context;
        inflater = (LayoutInflater) currentContext.getSystemService(Context.LAYOUT_INFLATER_SERVICE);

        currentView = inflater.inflate(R.layout.color_picker, this, true);

        initColorPicker(currentView);
    }

    private void initColorPicker(View currentView) {
        for (int i = 0; i < 7; i++) {
            ImageView tempColor = currentView.findViewById(colorPickers[i]);

            int tempI = i;
            tempColor.setOnClickListener(view -> setColorForWatermark(colors[tempI]));
        }
    }

    private void setColorForWatermark(int color) {
        textWatermark.setTextWatermarkColor(color);
    }

    public void setTextWatermark(TextWatermarkView textWatermark) {
        this.textWatermark = textWatermark;
    }

    public void show() {
        smoothScrollX(0);
    }

    public void hide() {
        smoothScrollX(getWidth());
    }
}
