package com.compdfkit.pdfviewer.customview.picker;

import android.content.Context;
import android.util.AttributeSet;
import android.view.LayoutInflater;
import android.view.View;
import android.widget.ImageView;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.constraintlayout.widget.ConstraintLayout;

import com.compdfkit.pdfviewer.R;

public class FontColorPicker extends ConstraintLayout {
    private View currentView;
    private LayoutInflater inflater;
    private Context currentContext;

    private TextView textView;

    private final int[] colorPickers = { R.id.color_1, R.id.color_2, R.id.color_3, R.id.color_4, R.id.color_5, R.id.color_6, R.id.color_7 };
    private final int[] colors = { 0xFFFF0000, 0xFFFF6600, 0xFFFFDD00, 0xFF00FF33, 0xFF00FFFF, 0xFF000000, 0xFF8800FF };

    public FontColorPicker(@NonNull Context context) {
        this(context, null);
    }

    public FontColorPicker(@NonNull Context context, @Nullable AttributeSet attrs) {
        this(context, attrs, 0);
    }

    public FontColorPicker(@NonNull Context context, @Nullable AttributeSet attrs, int defStyleAttr) {
        this(context, attrs, defStyleAttr, 0);
    }

    public FontColorPicker(@NonNull Context context, @Nullable AttributeSet attrs, int defStyleAttr, int defStyleRes) {
        super(context, attrs, defStyleAttr, defStyleRes);
        init(context);
    }

    private void init(Context context) {
        currentContext = context;
        inflater = (LayoutInflater) currentContext.getSystemService(Context.LAYOUT_INFLATER_SERVICE);

        currentView = inflater.inflate(R.layout.color_picker, this, true);

        initColorPicker(currentView);
    }

    private void initColorPicker(View currentView) {
        for (int i = 0; i < 7; i++) {
            ImageView tempColor = currentView.findViewById(colorPickers[i]);

            int tempI = i;
            tempColor.setOnClickListener(view -> textView.setTextColor(colors[tempI]));
        }
    }

    public void setTextView(TextView textView) {
        this.textView = textView;
    }
}
