package com.compdfkit.pdfviewer.customview.picker;

import android.content.Context;
import android.util.AttributeSet;
import android.view.LayoutInflater;
import android.view.View;
import android.widget.Button;
import android.widget.Scroller;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.constraintlayout.widget.ConstraintLayout;

import com.compdfkit.pdfviewer.R;

public class ImagePicker extends ConstraintLayout {
    private View currentView;
    private LayoutInflater inflater;
    private Context currentContext;

    private TextView imageSelectPathView;
    private Button imageSelectBtn;

    private Scroller mScroller;

    public ImagePicker(@NonNull Context context) {
        this(context, null);
    }

    public ImagePicker(@NonNull Context context, @Nullable AttributeSet attrs) {
        this(context, attrs, 0);
    }

    public ImagePicker(@NonNull Context context, @Nullable AttributeSet attrs, int defStyleAttr) {
        this(context, attrs, defStyleAttr, 0);
    }

    public ImagePicker(@NonNull Context context, @Nullable AttributeSet attrs, int defStyleAttr, int defStyleRes) {
        super(context, attrs, defStyleAttr, defStyleRes);
        this.mScroller = new Scroller(context);
        init(context);
    }

    @Override
    public void computeScroll() {
        super.computeScroll();
        if (mScroller.computeScrollOffset()) {
            scrollTo(mScroller.getCurrX(), mScroller.getCurrY());
            invalidate();
        }
    }

    public void smoothScrollX(int destX) {
        int scrollX = getScrollX();
        int deltaX = destX - scrollX;
        mScroller.startScroll(scrollX, 0, deltaX, 0, 500);
        invalidate();
    }

    private void init(Context context) {
        mScroller = new Scroller(context);

        currentContext = context;
        inflater = (LayoutInflater) currentContext.getSystemService(Context.LAYOUT_INFLATER_SERVICE);

        currentView = inflater.inflate(R.layout.image_picker, this, true);

        imageSelectPathView = currentView.findViewById(R.id.image_select_path);
        imageSelectBtn = currentView.findViewById(R.id.image_select_btn);
    }

    public TextView getImageSelectPathView() {
        return imageSelectPathView;
    }

    public Button getImageSelectBtn() {
        return imageSelectBtn;
    }

    public void show() {
        smoothScrollX(0);
    }

    public void hide() {
        smoothScrollX(getWidth());
    }
}
