package com.compdfkit.pdfviewer.fragments;

import android.annotation.SuppressLint;
import android.content.DialogInterface;
import android.os.Bundle;

import androidx.appcompat.app.AlertDialog;
import androidx.constraintlayout.widget.ConstraintLayout;
import androidx.fragment.app.Fragment;

import android.text.Editable;
import android.text.TextWatcher;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;

import com.compdfkit.pdfviewer.R;
import com.compdfkit.pdfviewer.activities.HeaderFooterActivity;
import com.compdfkit.pdfviewer.tools.DialogManager;
import com.compdfkit.pdfviewer.tools.HeaderFooterHandle;
import com.compdfkit.pdfviewer.tools.TextGenerator;
import com.google.android.material.button.MaterialButtonToggleGroup;
import com.google.android.material.dialog.MaterialAlertDialogBuilder;
import com.google.android.material.textfield.TextInputEditText;

import java.util.Objects;
import java.util.Random;

public class HeaderFooterConsoleFragment extends Fragment {
    private int type = 0;

    private HeaderFooterActivity activity;

    private static final String ARG_PARAM1 = "param1";
    private static final String ARG_PARAM2 = "param2";

    private String mParam1;
    private String mParam2;

    private MaterialButtonToggleGroup typeGroup;
    private Button commonTypeBtn;
    private Button batesTypeBtn;
    private Button typeSettingBtn;
    private TextInputEditText contentText;
    private Button addContentBtn;

    public HeaderFooterConsoleFragment() {
        // Required empty public constructor
    }

    public static HeaderFooterConsoleFragment newInstance(String param1, String param2) {
        HeaderFooterConsoleFragment fragment = new HeaderFooterConsoleFragment();
        Bundle args = new Bundle();
        args.putString(ARG_PARAM1, param1);
        args.putString(ARG_PARAM2, param2);
        fragment.setArguments(args);
        return fragment;
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        if (getArguments() != null) {
            mParam1 = getArguments().getString(ARG_PARAM1);
            mParam2 = getArguments().getString(ARG_PARAM2);
        }
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        activity = (HeaderFooterActivity) getActivity();
        View returnView = inflater.inflate(R.layout.fragment_header_footer_console, container, false);
        initViews(returnView);
        return returnView;
    }

    private void initViews(View returnView) {
        typeGroup = returnView.findViewById(R.id.type_group);
        commonTypeBtn = returnView.findViewById(R.id.common_btn);
        batesTypeBtn = returnView.findViewById(R.id.bates_btn);
        initTypeGroup();

        typeSettingBtn = returnView.findViewById(R.id.type_setting_btn);
        initTypeSettingBtn();

        contentText = returnView.findViewById(R.id.content_text);
        initContentText();

        addContentBtn = returnView.findViewById(R.id.add_content_btn);
        initAddContentBtn();
    }

    private void initTypeGroup() {
        typeGroup.addOnButtonCheckedListener((group, checkedId, isChecked) -> {
            if (isChecked) {
                if (checkedId == R.id.common_btn)
                    type = 0;
                else
                    type = 1;
            }
            activity.changeType(type);

            contentText.setText(HeaderFooterHandle.getSettingText(type, activity.getCurrentIndex()));
        });
    }

    private void initTypeSettingBtn() {
        typeSettingBtn.setOnClickListener(v -> {
            if (type == 0) {
                DialogManager.showCommonSettingDialog(getActivity(), activity.getCurrentIndex());
            } else {
                DialogManager.showBatesSettingDialog(getActivity(), activity.getCurrentIndex());
            }
        });
    }

    private void initContentText() {
        contentText.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence s, int start, int count, int after) {}

            @Override
            public void onTextChanged(CharSequence s, int start, int before, int count) {
                Log.d("DisplayTest", "display text:" + TextGenerator.convertBatesDisplayText(type, contentText.getText().toString()));
                HeaderFooterHandle.setSettingText(type, activity.getCurrentIndex(), Objects.requireNonNull(
                        type == 0 ? TextGenerator.convertCommonSettingText(type, contentText.getText().toString(), activity.getPageCount()) :
                        contentText.getText().toString()));
                activity.setHeaderFooterViewContent(activity.getCurrentIndex(),
                        type == 0 ? TextGenerator.convertCommonDisplayText(type, contentText.getText().toString(), activity.getPageCount()) :
                        TextGenerator.convertBatesDisplayText(type, contentText.getText().toString()));
            }

            @Override
            public void afterTextChanged(Editable s) {}
        });
    }

    @SuppressLint("SetTextI18n")
    private void initAddContentBtn() {
        addContentBtn.setOnClickListener(v -> {
            if (type == 0)
                DialogManager.showContentAppendDialog(getActivity(), activity.getCurrentIndex(), contentText);
            else {
                String content = contentText.getText().toString();
                String bates = TextGenerator.generateBatesText(
                        HeaderFooterHandle.getIndexPrefix(activity.getCurrentIndex()),
                        HeaderFooterHandle.getIndexSuffix(activity.getCurrentIndex()),
                        HeaderFooterHandle.getIndexDigits(activity.getCurrentIndex()),
                        HeaderFooterHandle.getIndexStartPageNumber(activity.getCurrentIndex()));

                contentText.setText(content + bates);
            }

        });
    }

    public void resetContentText(int index) {
        contentText.setText(HeaderFooterHandle.getSettingText(type, index));
    }
}