package com.compdfkit.pdfviewer.fragments;

import static android.app.Activity.RESULT_OK;

import android.Manifest;
import android.annotation.SuppressLint;
import android.content.ContentUris;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.database.Cursor;
import android.graphics.Bitmap;
import android.graphics.Matrix;
import android.net.Uri;
import android.os.Build;
import android.os.Bundle;

import androidx.annotation.Nullable;
import androidx.appcompat.app.AlertDialog;
import androidx.appcompat.widget.AppCompatEditText;
import androidx.appcompat.widget.AppCompatRadioButton;
import androidx.constraintlayout.widget.ConstraintLayout;
import androidx.core.app.ActivityCompat;
import androidx.core.content.ContextCompat;
import androidx.fragment.app.Fragment;

import android.provider.DocumentsContract;
import android.provider.MediaStore;
import android.text.Editable;
import android.text.TextWatcher;
import android.view.LayoutInflater;
import android.view.View;
import android.view.animation.AlphaAnimation;
import android.view.animation.Animation;
import android.widget.Button;
import android.widget.ImageButton;
import android.view.ViewGroup;
import android.widget.CheckBox;
import android.widget.TextView;

import com.compdfkit.core.common.CPDFDocumentException;
import com.compdfkit.pdfviewer.activities.WatermarkActivity;
import com.compdfkit.pdfviewer.R;
import com.compdfkit.pdfviewer.customview.picker.ColorPicker;
import com.compdfkit.pdfviewer.customview.picker.ImagePicker;
import com.compdfkit.pdfviewer.customview.watermarkview.ImageWatermarkView;
import com.compdfkit.pdfviewer.customview.watermarkview.TextWatermarkView;
import com.compdfkit.pdfviewer.customview.watermarkview.WatermarkView;
import com.compdfkit.pdfviewer.entity.WatermarkSetting;
import com.google.android.material.dialog.MaterialAlertDialogBuilder;
import com.google.android.material.slider.Slider;
import com.google.android.material.textfield.TextInputEditText;

import java.util.Objects;

public class WatermarkConsoleFragment extends Fragment {
    private int mode;
    private boolean isCustomRange = false;

    private float tWatermarkOpacity = 50;
    private float tWatermarkSize = 100;

    private float iWatermarkOpacity = 50;
    private float iWatermarkSize = 100;

    private int documentPageCount = 0;

    private float currentOpacity = 0.5f;

    private float hSpace = 100f;
    private float vSpace = 100f;

    private final WatermarkSetting setting = new WatermarkSetting();

    private WatermarkView associatedWatermark = null;

    private ColorPicker colorPicker;
    private ImagePicker imagePicker;

    private TextView imageSelectPathView;
    private Button imageSelectBtn;

    private Slider opacitySlider;
    private Slider sizeSlider;

    private TextView opacityText;
    private TextView sizeText;

    private TextView rangeMsg;
    private ImageButton rangeButton;

    private CheckBox fullScreenCheck;

    private TextInputEditText hSpaceEdit;
    private TextInputEditText vSpaceEdit;

    private String pageRange = "0";

    public WatermarkConsoleFragment() {
        // Required empty public constructor
    }

    public static WatermarkConsoleFragment newInstance(String param1, String param2) {
        WatermarkConsoleFragment fragment = new WatermarkConsoleFragment();
        Bundle args = new Bundle();
        fragment.setArguments(args);
        return fragment;
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        View returnView = inflater.inflate(R.layout.fragment_watermark_console, container, false);
        initViews(returnView);
        return returnView;
    }

    private void initViews(View returnView) {
        colorPicker = returnView.findViewById(R.id.color_picker);
        imagePicker = returnView.findViewById(R.id.image_picker);

        colorPicker.setTextWatermark((TextWatermarkView) associatedWatermark);

        imageSelectPathView = imagePicker.getImageSelectPathView();
        imageSelectBtn = imagePicker.getImageSelectBtn();
        initImageSelectBtn();

        opacityText = returnView.findViewById(R.id.opacity_text);
        opacitySlider = returnView.findViewById(R.id.opacity_slider);
        initOpacitySlider();

        sizeText = returnView.findViewById(R.id.size_text);
        sizeSlider = returnView.findViewById(R.id.size_slider);
        initSizeSlider();

        rangeMsg = returnView.findViewById(R.id.page_range_msg);
        rangeButton = returnView.findViewById(R.id.page_range_button);
        initRangeButton();

        fullScreenCheck = returnView.findViewById(R.id.full_screen_checkbox);
        initFullScreenCheck();

        hSpaceEdit = returnView.findViewById(R.id.horizontal_space);
        vSpaceEdit = returnView.findViewById(R.id.vertical_space);
        initSpaceEdits();
    }

    private void initImageSelectBtn() {
        imageSelectBtn.setOnClickListener(v -> {
            //动态申请获取访问 读写磁盘的权限
            if (ContextCompat.checkSelfPermission(getActivity(),
                    Manifest.permission.WRITE_EXTERNAL_STORAGE) != PackageManager.PERMISSION_GRANTED) {
                ActivityCompat.requestPermissions(getActivity(), new String[]{Manifest.permission.WRITE_EXTERNAL_STORAGE}, 101);
            } else {
                //打开相册
                if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.TIRAMISU) {
                    Intent intent = new Intent(MediaStore.ACTION_PICK_IMAGES);
                    startActivityForResult(intent, 102);
                } else {
                    Intent intent = new Intent(Intent.ACTION_GET_CONTENT);
                    //Intent.ACTION_GET_CONTENT = "android.intent.action.GET_CONTENT"
                    intent.setType("image/*");
                    startActivityForResult(intent, 102); // 打开相册
                }
            }
        });
    }

    @Override
    //TODO:Replace the deprecated method.
    public void onActivityResult(int requestCode, int resultCode, @Nullable Intent data) {
        super.onActivityResult(requestCode, resultCode, data);

        if (resultCode == RESULT_OK) {
            handleImageOnKitKat(data);
        }
    }

    private void handleImageOnKitKat(Intent data) {
        String imagePath = null;
        Uri uri = data.getData();
        if (DocumentsContract.isDocumentUri(getActivity(), uri)) {
            String docId = DocumentsContract.getDocumentId(uri);
            if ("com.android.providers.media.documents".equals(uri.getAuthority())) {
                String id = docId.split(":")[1];
                String selection = MediaStore.Images.Media._ID + "=" + id;
                imagePath = getImagePath(MediaStore.Images.Media.EXTERNAL_CONTENT_URI, selection);
            } else if ("com.android.providers.downloads.documents".equals(uri.getAuthority())) {
                Uri contentUri = ContentUris.withAppendedId(Uri.parse("content: //downloads/public_downloads"), Long.valueOf(docId));
                imagePath = getImagePath(contentUri, null);
            }
        } else if ("content".equalsIgnoreCase(uri.getScheme())) {
            imagePath = getImagePath(uri, null);
        } else if ("file".equalsIgnoreCase(uri.getScheme())) {
            imagePath = uri.getPath();
        }
        imageSelectPathView.setText(imagePath);
        ImageWatermarkView temp = (ImageWatermarkView) associatedWatermark;
        temp.setWatermarkImageFromPath(imagePath);
    }

    @SuppressLint("Range")
    private String getImagePath(Uri uri, String selection) {
        String path = null;

        Cursor cursor = getActivity().getContentResolver().query(uri, null, selection, null, null);
        if (cursor != null) {
            if (cursor.moveToFirst()) {
                path = cursor.getString(cursor.getColumnIndex(MediaStore.Images.Media.DATA));
            }
            cursor.close();
        }
        return path;
    }

    @SuppressLint("SetTextI18n")
    private void initOpacitySlider() {
        opacitySlider.addOnChangeListener((slider, value, fromUser) -> {
            currentOpacity = value / 100f;
            if (mode == 0)
                tWatermarkOpacity = value;
            else
                iWatermarkOpacity = value;

            opacityText.setText((int) value + "%");
            setting.setOpacity(value / 100);
            associatedWatermark.setWatermarkOpacity(value / 100);
        });
    }

    @SuppressLint("SetTextI18n")
    private void initSizeSlider() {
        sizeSlider.addOnChangeListener((slider, value, fromUser) -> {
            if (mode == 0)
                tWatermarkSize = value;
            else
                iWatermarkSize = value;

            sizeText.setText((int) value + "%");
            associatedWatermark.setWatermarkScale(value / 100);
        });
    }

    private void initRangeButton() {
        rangeButton.setOnClickListener(view -> {
            LayoutInflater inflater = LayoutInflater.from(getActivity());
            final ConstraintLayout rangeSelectLayout = (ConstraintLayout) inflater.inflate(R.layout.range_selector, null);
            final Button[] positiveButton = new Button[1];

            AppCompatRadioButton customRadioBtn = rangeSelectLayout.findViewById(R.id.custom_range);
            customRadioBtn.setChecked(isCustomRange);

            AppCompatEditText rangeEditText = rangeSelectLayout.findViewById(R.id.range_edit);
            rangeEditText.setEnabled(isCustomRange);
            if (isCustomRange)
                rangeEditText.setText(pageRange);


            MaterialAlertDialogBuilder inputDialog = new MaterialAlertDialogBuilder(getActivity());
            inputDialog.setTitle(R.string.setting_range).setView(rangeSelectLayout);
            inputDialog.setPositiveButton(R.string.done, (dialogInterface, i) -> {
                if (customRadioBtn.isChecked()) {
                    pageRange = Objects.requireNonNull(rangeEditText.getText()).toString();
                    rangeMsg.setText(pageRange);
                } else {
                    //TODO: Repair the code to set All pages range
                    rangeMsg.setText(R.string.all_pages);
                    pageRange = "0-" + (documentPageCount - 1);
                }
                isCustomRange = customRadioBtn.isChecked();
            });

            inputDialog.setNegativeButton(R.string.cancel, (dialogInterface, i) -> {});

            customRadioBtn.setOnCheckedChangeListener((compoundButton, b) -> {
                rangeEditText.setEnabled(b);
                if (b && Objects.requireNonNull(rangeEditText.getText()).toString().equals(""))
                    positiveButton[0].setEnabled(false);
            });

            rangeEditText.addTextChangedListener(new TextWatcher() {
                @Override
                public void beforeTextChanged(CharSequence charSequence, int i, int i1, int i2) {}

                @Override
                public void onTextChanged(CharSequence charSequence, int i, int i1, int i2) {}

                @Override
                public void afterTextChanged(Editable editable) {
                    positiveButton[0].setEnabled(!rangeEditText.getText().toString().equals(""));
                }
            });

            AlertDialog dialog = inputDialog.create();
            dialog.show();
            positiveButton[0] = dialog.getButton(DialogInterface.BUTTON_POSITIVE);
        });
    }

    private void initFullScreenCheck() {
        WatermarkActivity activity = (WatermarkActivity) getActivity();

        fullScreenCheck.setOnCheckedChangeListener((compoundButton, b) -> {
            boolean isFullScreen = fullScreenCheck.isChecked();

            opacitySlider.setEnabled(!isFullScreen);
            sizeSlider.setEnabled(!isFullScreen);

            hSpaceEdit.setEnabled(isFullScreen);
            vSpaceEdit.setEnabled(isFullScreen);

            assert activity != null;
            activity.showFullScreenMask(b);

//            associatedWatermark.setVisibility(b ? View.INVISIBLE : View.VISIBLE);
        });
    }

    private void initSpaceEdits() {
        hSpaceEdit.setOnFocusChangeListener(new View.OnFocusChangeListener() {
            @Override
            public void onFocusChange(View view, boolean b) {
                if (!b) {
                    sethSpace(Float.parseFloat(Objects.requireNonNull(hSpaceEdit.getText()).toString()));
                    WatermarkActivity activity = (WatermarkActivity) getActivity();
                    assert activity != null;
                    activity.showFullScreenMask(true);
                }
            }
        });

        vSpaceEdit.setOnFocusChangeListener((view, b) -> {
            if (!b) {
                setvSpace(Float.parseFloat(Objects.requireNonNull(vSpaceEdit.getText()).toString()));
                WatermarkActivity activity = (WatermarkActivity) getActivity();
                assert activity != null;
                activity.showFullScreenMask(true);
            }
        });
    }

    public void setAssociatedWatermark(WatermarkView watermark) {
        this.associatedWatermark = watermark;
    }

    public void changePicker(int mode) {
        this.mode = mode;

        AlphaAnimation appearAnimation = new AlphaAnimation(0, 1);
        appearAnimation.setDuration(250);

        AlphaAnimation disappearAnimation = new AlphaAnimation(1, 0);
        disappearAnimation.setDuration(250);

        if (mode == 0) {
            disappearAnimation.setAnimationListener(new Animation.AnimationListener() {
                @Override
                public void onAnimationStart(Animation animation) {
                    colorPicker.setVisibility(View.VISIBLE);
                }

                @Override
                public void onAnimationEnd(Animation animation) {
                    imagePicker.setVisibility(View.INVISIBLE);
                }

                @Override
                public void onAnimationRepeat(Animation animation) {

                }
            });

            colorPicker.startAnimation(appearAnimation);
            imagePicker.startAnimation(disappearAnimation);
        } else {
            disappearAnimation.setAnimationListener(new Animation.AnimationListener() {
                @Override
                public void onAnimationStart(Animation animation) {
                    imagePicker.setVisibility(View.VISIBLE);
                }

                @Override
                public void onAnimationEnd(Animation animation) {
                    colorPicker.setVisibility(View.INVISIBLE);
                }

                @Override
                public void onAnimationRepeat(Animation animation) {

                }
            });

            colorPicker.startAnimation(disappearAnimation);
            imagePicker.startAnimation(appearAnimation);
        }

        currentOpacity = (this.mode == 0 ? tWatermarkOpacity : iWatermarkOpacity) / 100.0f;
        opacitySlider.setValue(this.mode == 0 ? tWatermarkOpacity : iWatermarkOpacity);
        sizeSlider.setValue(this.mode == 0 ? tWatermarkSize : iWatermarkSize);
        if (fullScreenCheck.isChecked()) {
            WatermarkActivity activity = (WatermarkActivity) getActivity();
            assert activity != null;
            activity.showFullScreenMask(true);
        }
    }

    public WatermarkSetting getWatermarkSetting() {
        setting.setOpacity(opacitySlider.getValue() / 100);
        setting.setFullScreen(fullScreenCheck.isChecked());
        setting.setPages(pageRange);
        if (fullScreenCheck.isChecked()) {
            setting.setHorizontalSpacing(Float.parseFloat(Objects.requireNonNull(hSpaceEdit.getText()).toString()));
            setting.setVerticalSpacing(Float.parseFloat(Objects.requireNonNull(vSpaceEdit.getText()).toString()));
        }
        return setting;
    }

    public float getCurrentOpacity() {
        return currentOpacity;
    }

    public float gethSpace() {
        return Float.parseFloat(Objects.requireNonNull(hSpaceEdit.getText()).toString());
    }

    public void sethSpace(float hSpace) {
        this.hSpace = hSpace;
    }

    public float getvSpace() {
        return Float.parseFloat(Objects.requireNonNull(vSpaceEdit.getText()).toString());
    }

    public void setvSpace(float vSpace) {
        this.vSpace = vSpace;
    }

    public int getMode() {
        return mode;
    }

    public void setDocumentPageCount(int count) {
        this.documentPageCount = count;
        pageRange = "0-" + (count - 1);
    }

    public void resetWatermarkFromSetting(int index, WatermarkSetting setting) throws CPDFDocumentException {
        WatermarkActivity activity = (WatermarkActivity) getActivity();
        assert activity != null;

        activity.closeWatermarkList();
        activity.hideWatermark(index);

        activity.changeWatermark(setting.getType());

        if (mode == 0) {
            TextWatermarkView temp = (TextWatermarkView) associatedWatermark;
            temp.setWatermarkTextContent(setting.getText());
            temp.setTextWatermarkColor(setting.getTextRGBColor());
            temp.setWatermarkTextFontSize(setting.getFontSize());
        } else {
            ImageWatermarkView temp = (ImageWatermarkView) associatedWatermark;
            Bitmap settingBitmap = setting.getImage();
            Matrix matrix = new Matrix();
            matrix.postScale(temp.getSizeConvertRate(), temp.getSizeConvertRate());

            settingBitmap = Bitmap.createBitmap(settingBitmap, 0, 0, settingBitmap.getWidth(), settingBitmap.getHeight(), matrix, true);

            temp.setWatermarkImageContent(settingBitmap);
        }

        associatedWatermark.setWatermarkOpacity(setting.getOpacity());
        opacitySlider.setValue((int) (setting.getOpacity() * 100));

        associatedWatermark.setRotationRadian(setting.getRotation());
        associatedWatermark.setRotation((float) -Math.toDegrees(setting.getRotation()));

        activity.changeWatermarkLocation(setting.getHorizOffset(), setting.getVertOffset());

        fullScreenCheck.setChecked(setting.getFullScreen());
        hSpaceEdit.setText(String.valueOf(setting.getHorizontalSpacing()));
        vSpaceEdit.setText(String.valueOf(setting.getVerticalSpacing()));

        isCustomRange = !setting.getPages().equals("0-" + (documentPageCount - 1));
        pageRange = setting.getPages();

        if (isCustomRange) {
            rangeMsg.setText(pageRange);
        } else {
            rangeMsg.setText(R.string.all_pages);
        }
    }
}

