package com.compdfkit.pdfviewer.tools;

import android.graphics.RectF;

import com.compdfkit.core.document.CPDFBates;
import com.compdfkit.core.document.CPDFHeaderFooter;
import com.compdfkit.pdfviewer.entity.HeaderFooterSetting;

import java.util.ArrayList;

public class HeaderFooterHandle {
    private final static HeaderFooterSetting commonTypeSetting = new HeaderFooterSetting();
    private final static HeaderFooterSetting batesTypeSetting = new HeaderFooterSetting();

    private static int[] dateFormatIndexs = new int[] { 0, 0, 0, 0, 0, 0 };
    private static int[] pageFormatIndexs = new int[] { 0, 0, 0, 0, 0, 0 };
    private static int[] startPageNumbers = new int[] { 1, 1, 1, 1, 1, 1 };

    private static int[] digits = new int[] { 1, 1, 1, 1, 1, 1 };
    private static String[] prefixs = new String[] { "", "", "", "", "", "" };
    private static String[] suffixs = new String[] { "", "", "", "", "", "" };

    public static void updateCommonType(CPDFHeaderFooter headerFooter) {
        RectF margin = commonTypeSetting.getMargin();
        headerFooter.setMargin(margin.left, margin.top, margin.right, margin.bottom);
        headerFooter.setPageOffset(commonTypeSetting.getPageOffset());
        headerFooter.setPages(commonTypeSetting.getPages());

        for (int i = 0; i < 6; i++) {
            headerFooter.setFontName(i, commonTypeSetting.getFontName(i));
            headerFooter.setFontSize(i, commonTypeSetting.getFontSize(i));
            headerFooter.setText(i, commonTypeSetting.getText(i));
//            headerFooter.setText(i, "<<Page 1>>");
            headerFooter.setTextColor(i, commonTypeSetting.getColor(i));
        }

        headerFooter.update();
    }

    public static void updateBatesType(CPDFBates bates) {
        RectF margin = batesTypeSetting.getMargin();
        bates.setMargin(margin.left, margin.top, margin.right, margin.bottom);
        bates.setPageOffset(batesTypeSetting.getPageOffset());
        bates.setPages(batesTypeSetting.getPages());

        for (int i = 0; i < 6; i++) {
            bates.setFontName(i, batesTypeSetting.getFontName(i));
            bates.setFontSize(i, batesTypeSetting.getFontSize(i));
            bates.setText(i, batesTypeSetting.getText(i));
            bates.setTextColor(i, batesTypeSetting.getColor(i));
        }

        bates.update();
    }

    private static HeaderFooterSetting getSettingFromType(int type) {
        return type == 0 ? commonTypeSetting : batesTypeSetting;
    }

    public static void setSettingMargin(int type, RectF margin) {
        getSettingFromType(type).setMargin(margin);
    }

    public static RectF getSettingMargin(int type) {
        return getSettingFromType(type).getMargin();
    }

    public static void setSettingPageOffset(int type, int pageOffset) {
        getSettingFromType(type).setPageOffset(pageOffset);
    }

    public static int getSettingPageOffset(int type) {
        return getSettingFromType(type).getPageOffset();
    }

    public static void setSettingPages(int type, String pages) {
        getSettingFromType(type).setPages(pages);
    }

    public static String getSettingPages(int type) {
        return getSettingFromType(type).getPages();
    }

    public static void setSettingFontName(int type, int index, String fontName) {
        getSettingFromType(type).setFontName(index, fontName);
    }

    public static String getSettingFontName(int type, int index) {
        return getSettingFromType(type).getFontName(index);
    }

    public static void setSettingFontSize(int type, int index, float fontSize) {
        getSettingFromType(type).setFontSize(index, fontSize);
    }

    public static float getSettingFontSize(int type, int index) {
        return getSettingFromType(type).getFontSize(index);
    }

    public static void setSettingText(int type, int index, String text) {
        getSettingFromType(type).setText(index, text);
    }

    public static String getSettingText(int type, int index) {
        return getSettingFromType(type).getText(index);
    }

    public static void setSettingColor(int type, int index, int color) {
        getSettingFromType(type).setColor(index, color);
    }

    public static int getSettingColor(int type, int index) {
        return getSettingFromType(type).getColor(index);
    }

    public static void setIndexDateFormat(int index, int dateFormatIndex) {
        dateFormatIndexs[index] = dateFormatIndex;
    }

    public static int getIndexDateFormat(int index) {
        return dateFormatIndexs[index];
    }

    public static void setIndexPageFormat(int index, int pageFormatIndex) {
        pageFormatIndexs[index] = pageFormatIndex;
    }

    public static int getIndexPageFormat(int index) {
        return pageFormatIndexs[index];
    }

    public static void setIndexStartPageNumber(int index, int startPageNumber) {
        startPageNumbers[index] = startPageNumber;
    }

    public static int getIndexStartPageNumber(int index) {
        return startPageNumbers[index];
    }

    public static void setIndexDigits(int index, int digit) {
        digits[index] = digit;
    }

    public static int getIndexDigits(int index) {
        return digits[index];
    }

    public static void setIndexPrefix(int index, String prefix) {
        prefixs[index] = prefix;
    }

    public static String getIndexPrefix(int index) {
        return prefixs[index];
    }

    public static void setIndexSuffix(int index, String suffix) {
        suffixs[index] = suffix;
    }

    public static String getIndexSuffix(int index) {
        return suffixs[index];
    }
}
