package com.compdfkit.pdfviewer.tools;

import android.annotation.SuppressLint;
import android.util.Log;

import com.compdfkit.pdfviewer.R;

import java.text.SimpleDateFormat;
import java.util.Arrays;
import java.util.Date;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class TextGenerator {
    private static final String[] dateFormatParseStrings = new String[] {
            "m/d",
            "m/d/yy",
            "m/d/yyyy",
            "d/m/yy",
            "d/m/yyyy",
            "dd/mm/yy",
            "dd/mm/yyyy",
            "mm/yy",
            "mm/yyyy",
            "m.d.yy",
            "m.d.yyyy",
            "mm.dd.yy",
            "mm.dd.yyyy",
            "mm.yy",
            "mm.yyyy",
            "d.m.yy",
            "d.m.yyyy",
            "dd.mm.yy",
            "dd.mm.yyyy",
            "yy-mm-dd",
            "yyyy-mm-dd" };

    private static final String[] pageParseStrings = new String[] {
            "1",
            "1 of n",
            "1/n",
            "Page 1",
            "Page 1 of n"};

    public static String generateBatesText(String prefix, String suffix, int digits, int startNumber) {
        String text = "<<";
        text += "#";
        text += digits;
        text += "#";
        text += startNumber;

        text += "#";
        if (prefix != null && !prefix.equals("")) {
            text += prefix;
        }
        text += "#";
        if (suffix != null && !suffix.equals("")) {
            text += suffix;
        }

        text += ">>";

        return text;
    }

    public static String generateDateText(String dateFormatString) {
        Date now = new Date();

        @SuppressLint("SimpleDateFormat")
        SimpleDateFormat dateFormat = new SimpleDateFormat(dateFormatString);

        return dateFormat.format(now);
    }

    public static String convertCommonDisplayText(int type, String contentText, int pageCount) {
        String displayText = contentText;
        String pair = "<<[ Pageofnmdy0-9/\\-.]*>>";
        Pattern r = Pattern.compile(pair);

        Matcher m = r.matcher(contentText);
        while (m.find()) {
            Log.d("Match", "convertContentText: " + convertCommonDisplayText(m.group(), pageCount));
            displayText = displayText.replaceAll(m.group(), convertCommonDisplayText(m.group(), pageCount));
        }

        return displayText;
    }

    public static String convertCommonDisplayText(String text, int pageCount) {
        String textContent = text.substring(2, text.length() - 2);
        String displayText = text;

        if (Arrays.asList(pageParseStrings).contains(textContent)) {
            displayText = textContent.replaceAll("1", "1");//TODO: replace the display page
            displayText = displayText.replaceAll("n", String.valueOf(pageCount));
        } else if (Arrays.asList(dateFormatParseStrings).contains(textContent)) {
            String dateFormat = textContent.replaceAll("m", "M");
            displayText = generateDateText(dateFormat);
        }

        return displayText;
    }

    public static String convertCommonSettingText(int type, String contentText, int pageCount) {
        String settingText = contentText;
        String pair = "<<[ Pageofnmdy0-9/\\-.]*>>";
        Pattern r = Pattern.compile(pair);

        Matcher m = r.matcher(contentText);
        while (m.find()) {
            Log.d("Match", "convertContentText: " + convertCommonSettingText(m.group(), pageCount));
            settingText = settingText.replaceAll(m.group(), convertCommonSettingText(m.group(), pageCount));
        }

        return settingText;
    }

    public static String convertCommonSettingText(String text, int pageCount) {
        String textContent = text.substring(2, text.length() - 2);
        String settingText = text;

        if (Arrays.asList(pageParseStrings).contains(textContent)) {
            settingText = textContent.replaceAll("1", "<<1>>");
            settingText = settingText.replaceAll("n", String.valueOf(pageCount));
        } else if (Arrays.asList(dateFormatParseStrings).contains(textContent)) {
            String dateFormat = textContent.replaceAll("m", "M");
            settingText = generateDateText(dateFormat);
        }

        return settingText;
    }

    public static String convertBatesDisplayText(int type, String contentText) {
        String settingText = contentText;
        String pair = "<<#.*#.*#.*#.*>>";
        Pattern r = Pattern.compile(pair);

        Matcher m = r.matcher(contentText);
        while (m.find()) {
            Log.d("Match", "convertContentText: " + convertBatesDisplayText(m.group()));
            settingText = settingText.replaceAll(m.group(), convertBatesDisplayText(m.group()));
        }

        return settingText;
    }

    public static String convertBatesDisplayText(String text) {
        String textContent = text.substring(2, text.length() - 2);
        StringBuilder settingText = new StringBuilder();

        String[] componentTexts = textContent.split("#", -1);

        //The first item is a "", so index + 1;
        settingText.append(componentTexts[3]);

        if (!componentTexts[1].equals("")) {
            int number = componentTexts[2].equals("") ? 1 : Integer.parseInt(componentTexts[2]);
            int digits = Integer.parseInt(componentTexts[1]);

            while (number > 0) {
                digits--;
                number /= 10;
            }

            for (int i = 0; i < digits; i++)
                settingText.append("0");
        }

        if (componentTexts[2].equals(""))
            settingText.append("1");
        else
            settingText.append(componentTexts[2]);

        settingText.append(componentTexts[4]);

        return settingText.toString();
    }
}
